@@ -15,18 +15,17 @@ import logging
 import sys
 import warnings
 
+import django
 from django.conf import settings
 from django.core.signals import got_request_exception, request_started
 from threading import Lock
 
-from raven._compat import PY2, binary_type, text_type
 from raven.utils.conf import convert_options
+from raven.utils.compat import PY2, binary_type, text_type
 from raven.utils.imports import import_string
 
 logger = logging.getLogger('sentry.errors.client')
 
-settings_lock = Lock()
-
 
 def get_installed_apps():
     """
@@ -108,12 +107,6 @@ class ProxyClient(object):
 client = ProxyClient()
 
 
-def get_option(x, d=None):
-    options = getattr(settings, 'RAVEN_CONFIG', {})
-
-    return getattr(settings, 'SENTRY_%s' % x, options.get(x, d))
-
-
 def get_client(client=None, reset=False):
     global _client
 
@@ -173,7 +166,13 @@ class SentryDjangoHandler(object):
             SentryCeleryHandler, register_logger_signal
         )
 
-        self.celery_handler = SentryCeleryHandler(client).install()
+        ignore_expected = getattr(settings,
+                                  'SENTRY_CELERY_IGNORE_EXPECTED',
+                                  False)
+
+        self.celery_handler = SentryCeleryHandler(client,
+                                                  ignore_expected=ignore_expected)\
+                                                  .install()
 
         # try:
         #     ga = lambda x, d=None: getattr(settings, 'SENTRY_%s' % x, d)
@@ -220,36 +219,60 @@ def register_serializers():
     import raven.contrib.django.serializers  # NOQA
 
 
-def install_middleware():
+def install_middleware(middleware_name, lookup_names=None):
     """
-    Force installation of SentryMiddlware if it's not explicitly present.
-
-    This ensures things like request context and transaction names are made
-    available.
+    Install specified middleware
     """
-    name = 'raven.contrib.django.middleware.SentryMiddleware'
-    all_names = (name, 'raven.contrib.django.middleware.SentryLogMiddleware')
-    with settings_lock:
-        # default settings.MIDDLEWARE is None
-        middleware_attr = 'MIDDLEWARE' if getattr(settings,
-                                                  'MIDDLEWARE',
-                                                  None) is not None \
-            else 'MIDDLEWARE_CLASSES'
-        # make sure to get an empty tuple when attr is None
-        middleware = getattr(settings, middleware_attr, ()) or ()
-        if set(all_names).isdisjoint(set(middleware)):
-            setattr(settings,
-                    middleware_attr,
-                    type(middleware)((name,)) + middleware)
+    if lookup_names is None:
+        lookup_names = (middleware_name,)
+    # default settings.MIDDLEWARE is None
+    middleware_attr = 'MIDDLEWARE' if getattr(settings,
+                                              'MIDDLEWARE',
+                                              None) is not None \
+        else 'MIDDLEWARE_CLASSES'
+    # make sure to get an empty tuple when attr is None
+    middleware = getattr(settings, middleware_attr, ()) or ()
+    if set(lookup_names).isdisjoint(set(middleware)):
+        setattr(settings,
+                middleware_attr,
+                type(middleware)((middleware_name,)) + middleware)
 
 
-if (
-    'raven.contrib.django' in settings.INSTALLED_APPS or
-    'raven.contrib.django.raven_compat' in settings.INSTALLED_APPS
-):
-    register_serializers()
-    install_middleware()
+_setup_lock = Lock()
+
+_initialized = False
+
+def initialize():
+    global _initialized
+
+    with _setup_lock:
+        if _initialized:
+            return
+
+        # mark this as initialized immediatley to avoid recursive import issues
+        _initialized = True
+
+        try:
+            register_serializers()
+            install_middleware(
+                'raven.contrib.django.middleware.SentryMiddleware',
+                (
+                    'raven.contrib.django.middleware.SentryMiddleware',
+                    'raven.contrib.django.middleware.SentryLogMiddleware'))
+            install_middleware(
+                'raven.contrib.django.middleware.DjangoRestFrameworkCompatMiddleware')
+
+            # XXX(dcramer): maybe this setting should disable ALL of this?
+            if not getattr(settings, 'DISABLE_SENTRY_INSTRUMENTATION', False):
+                handler = SentryDjangoHandler()
+                handler.install()
+
+            # instantiate client so hooks get registered
+            get_client()  # NOQA
+        except Exception:
+            _initialized = False
+
+# Django 1.7 uses ``raven.contrib.apps.RavenConfig``
+if django.VERSION < (1, 7, 0):
+    initialize()
 
-    if not getattr(settings, 'DISABLE_SENTRY_INSTRUMENTATION', False):
-        handler = SentryDjangoHandler()
-        handler.install()
